/*

    File: file_acb.c

    Copyright (C) 2012 Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <stdio.h>
#include "types.h"
#include "filegen.h"

static void register_header_check_acb(file_stat_t *file_stat);

const file_hint_t file_hint_acb= {
  .extension="acb",
  .description="Adobe Color Book",
  .min_header_distance=0,
  .max_filesize=100*1024*1024,
  .recover=1,
  .enable_by_default=1,
  .register_header_check=&register_header_check_acb
};

static int header_check_acb(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
{
  reset_file_recovery(file_recovery_new);
  file_recovery_new->extension=file_hint_acb.extension;
  return 1;
}

static void register_header_check_acb(file_stat_t *file_stat)
{
  /* http://magnetiq.com/pages/acb-spec/
   * magic:	8BCB
   * version:	1
   * identifier	0xB.. */
  static const unsigned char acb_header[7]=  {
    '8' , 'B' , 'C' , 'B' , 0x00, 0x01, 0x0b
  };
  register_header_check(0, acb_header, sizeof(acb_header), &header_check_acb, file_stat);
}
